==> alihbase-connector-1.0.4-sources.jar: com/alibaba/hbase/client/AliHBaseUEConnection.java <==
/*
 * Copyright Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.hbase.client;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;

import com.alibaba.lindorm.client.AdminService;
import com.alibaba.lindorm.client.LindormClientConfig;
import com.alibaba.lindorm.client.LindormClientConstants;
import com.alibaba.lindorm.client.LindormServiceProvider;
import com.alibaba.lindorm.client.SystemService;
import com.alibaba.lindorm.client.TableService;
import com.alibaba.lindorm.client.WideColumnService;
import com.alibaba.lindorm.client.core.utils.ClientEnvLogUtil;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HConstants;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.BufferedMutator;
import org.apache.hadoop.hbase.client.BufferedMutatorParams;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.RegionLocator;
import org.apache.hadoop.hbase.client.Table;
import org.apache.hadoop.hbase.security.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AliHBaseUEConnection  implements Connection {
  private static final Logger LOG = LoggerFactory.getLogger(AliHBaseUEConnection.class);


  private Configuration conf;


  private boolean isClosed = false;
  private boolean isAborted = false;

  private String seedServer;
  private String userName;
  private String password;

  private int operationTimeout;
  private int connectTimeout;

  private boolean accessFromInternet;
  private ConcurrentHashMap<String, WideColumnService> wideColumnMap = new ConcurrentHashMap<>();
  private ConcurrentHashMap<String, TableService> tableServiceMap = new ConcurrentHashMap<>();
  private AdminService adminService;
  private SystemService systemService;

  private LindormClientConfig lindormClientConfig;


  static {
    ClientEnvLogUtil.setEnabled(false);
  }

  // For HBase-1.x
  public AliHBaseUEConnection(Configuration conf, boolean managed, ExecutorService pool,
      final User user) throws IOException {
    this(conf, pool, user);
  }

  public AliHBaseUEConnection(Configuration conf, ExecutorService pool, final User user) throws IOException {
    this.conf = conf;
    String host = conf.get(Constants.ALIHBASE_SERVER_NAME);
    if (host == null || host.trim().isEmpty()) {
      this.seedServer = conf.get(LindormClientConstants.SEED_SERVERS);
    } else {
      if (host.contains(":")) {
        // the port is set with the host name
        this.seedServer = host;
      } else {
        int port = Constants.DEFAULT_ALIHBASEUE_PORT;
        this.seedServer = host + ":" + port;
      }
    }

    this.operationTimeout = conf.getInt(HConstants.HBASE_CLIENT_OPERATION_TIMEOUT,
        HConstants.DEFAULT_HBASE_CLIENT_OPERATION_TIMEOUT);
    this.connectTimeout = conf.getInt(Constants.SOCKET_TIMEOUT_CONNECT, Constants.DEFAULT_SOCKET_TIMEOUT_CONNECT);

    this.userName = conf.get(Constants.USERNAME);
    this.password = conf.get(Constants.PASSWORD);

    String publicIdentifier = conf.get(Constants.PUBLIC_HOSTNAME_IDENTIFIER, Constants.DEFAULT_PUBLIC_HOSTNAME_IDENTIFIER);

    if (seedServer.contains(publicIdentifier)) {
      this.accessFromInternet = true;
    }
    if (conf.get(Constants.ALIHBASE_ACCESS_FROM_INTERNET) != null) {
      this.accessFromInternet = conf.getBoolean(Constants.ALIHBASE_ACCESS_FROM_INTERNET, Constants.ALIHBASE_ACCESS_FROM_INTERNET_DEFAULT);

    }
  }


  @Override
  public Configuration getConfiguration() {
    return conf;
  }

  @Override
  public BufferedMutator getBufferedMutator(TableName tableName) throws IOException {
    return new AliHBaseUEBufferedMutator(AliHBaseUEConnection.this, tableName);
  }

  @Override
  public BufferedMutator getBufferedMutator(BufferedMutatorParams params)
      throws IOException {
    if (params.getTableName() == null) {
      throw new IllegalArgumentException("TableName cannot be null.");
    } else {
      return new AliHBaseUEBufferedMutator(AliHBaseUEConnection.this, params.getTableName());
    }
  }

  @Override
  public RegionLocator getRegionLocator(TableName tableName) throws IOException {
    return new AliHBaseUETable(tableName, AliHBaseUEConnection.this);
  }

  @Override
  public Admin getAdmin() throws IOException {
    return new AliHBaseUEAdmin(this);
  }

  @Override
  public synchronized void close() throws IOException {
    if (isClosed) {
      return;
    }
    this.isClosed = true;
    try {
      if (adminService != null) {
        adminService.close();
      }
    } catch (IOException ioE) {
      LOG.error("Error happened when closing adminservice", ioE);
    }
    try {
      if (systemService != null) {
        systemService.close();
      }
    } catch (Throwable e) {
      LOG.error("Error happened when closing systemService", e);
    }
    for (Map.Entry<String, WideColumnService> entry : wideColumnMap.entrySet()) {
      if (entry.getValue() != null) {
        try {
          entry.getValue().close();
        } catch (IOException ioE) {
          LOG.error("Error happened when closing WideColumnService for namespace " + entry.getKey(), ioE);
        }
      }
    }
  }

  @Override
  public boolean isClosed() {
    return isClosed;
  }

  @Override
  public Table getTable(TableName tableName) throws IOException {
    return new AliHBaseUETable(tableName, AliHBaseUEConnection.this);
  }

  public Table getTable(String tableName) throws IOException {
    return getTable(TableName.valueOf(tableName));
  }

  @Override
  public Table getTable(TableName tableName, ExecutorService executorService) throws IOException {
    return getTable(tableName, null);
  }

  @Override
  public void abort(String s, Throwable throwable) {
    isAborted = true;
    try {
      close();
    } catch (IOException ioE) {
      LOG.error("Error happened when aborting", ioE);
    }
  }

  @Override
  public boolean isAborted() {
    return isAborted;
  }

  public int getOperationTimeout() {
    return operationTimeout;
  }

  AdminService getAdminService() throws IOException {
    if (isClosed()) {
      throw new IOException("Connection is closed");
    }
    if (adminService == null) {
      synchronized (this) {
        if (isClosed()) {
          throw new IOException("Connection is closed");
        }
        if (adminService == null) {
          adminService = LindormServiceProvider.AdminServiceProvider.create(getLindormConfig());
        }
      }
    }
    return adminService;
  }

  SystemService getSystemService() throws IOException {
    if (isClosed()) {
      throw new IOException("Connection is closed");
    }
    if (systemService == null) {
      synchronized (this) {
        if (isClosed()) {
          throw new IOException("Connection is closed");
        }
        if (systemService == null) {
          systemService = LindormServiceProvider.SystemServiceProvider.create(getLindormConfig());
        }
      }
    }
    return systemService;
  }

  WideColumnService getWideColumnService(String namespace) throws IOException {
    if (isClosed()) {
      throw new IOException("Connection is closed");
    }
    if (namespace == null) {
      throw new IOException("namespace can't be null!");
    }
    WideColumnService wideColumnService = wideColumnMap.get(namespace);
    if (wideColumnService == null) {
      synchronized (this) {
        if (isClosed()) {
          throw new IOException("Connection is closed");
        }
        LindormClientConfig config = getLindormConfig();
        config.setNamespace(namespace);
        WideColumnService service = LindormServiceProvider.WideColumnServiceProvider.create(config);
        WideColumnService old = wideColumnMap.putIfAbsent(namespace, service);
        if (old != null) {
          service.close();
          return old;
        } else {
          return service;
        }
      }

    } else {
      return wideColumnService;
    }
  }



  private LindormClientConfig createLindormConfig() throws IOException {
    LindormClientConfig lindormClientConfig = LindormClientConfig.create();
    Iterator<Map.Entry<String, String>> it = conf.iterator();
    while (it.hasNext()) {
      Map.Entry<String, String> entry = it.next();
      if (entry.getKey() != null && entry.getValue() != null) {
        lindormClientConfig.set(entry.getKey(), entry.getValue());
      }
    }
    if (userName != null && !userName.isEmpty() && password != null && !password.isEmpty()) {
      lindormClientConfig.setUser(userName, password);
    }
    lindormClientConfig.setSeedServer(seedServer.trim());
    lindormClientConfig.setTimeout(operationTimeout);
    if (accessFromInternet) {
      lindormClientConfig.setBoolean(LindormClientConstants.LINDOM_RPC_ONLY_USE_SEEDSERVER, true);
    }
    if (conf.get(LindormClientConstants.CLIENT_SORTER_TYPE) != null) {
      lindormClientConfig.set(LindormClientConstants.CLIENT_SORTER_TYPE,
          conf.get(LindormClientConstants.CLIENT_SORTER_TYPE));
    } else {
      lindormClientConfig
          .set(LindormClientConstants.CLIENT_SORTER_TYPE, LindormClientConstants.CLIENT_NAME_SORTER);
    }
    return lindormClientConfig;
  }

  LindormClientConfig getLindormConfig() throws IOException {
    if (lindormClientConfig == null) {
      synchronized (this) {
        if (lindormClientConfig == null) {
          lindormClientConfig = createLindormConfig();
        }
      }
    }
    return new LindormClientConfig(lindormClientConfig);
  }

  public TableService getTableServiceService(String namespace) throws IOException {
    if (isClosed()) {
      throw new IOException("Connection is closed");
    }
    if (namespace == null) {
      throw new IOException("namespace can't be null!");
    }
    TableService tableService = tableServiceMap.get(namespace);
    if (tableService == null) {
      synchronized (this) {
        if (isClosed()) {
          throw new IOException("Connection is closed");
        }
        LindormClientConfig config = getLindormConfig();
        config.setNamespace(namespace);
        TableService service = LindormServiceProvider.TableServiceProvider.create(config);
        TableService old = tableServiceMap.putIfAbsent(namespace, service);
        if (old != null) {
          service.close();
          return old;
        } else {
          return service;
        }
      }

    } else {
      return tableService;
    }
  }
}

==> alihbase-connector-1.0.9-sources.jar: com/alibaba/hbase/client/AliHBaseUEConnection.java <==
/*
 * Copyright Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.hbase.client;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;

import com.alibaba.lindorm.client.AdminService;
import com.alibaba.lindorm.client.LindormClientConfig;
import com.alibaba.lindorm.client.LindormClientConstants;
import com.alibaba.lindorm.client.LindormServiceProvider;
import com.alibaba.lindorm.client.SystemService;
import com.alibaba.lindorm.client.TableService;
import com.alibaba.lindorm.client.WideColumnService;
import com.alibaba.lindorm.client.core.utils.ClientEnvLogUtil;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HConstants;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.AliHBaseUEAdmin;
import org.apache.hadoop.hbase.client.AliHBaseUETable;
import org.apache.hadoop.hbase.client.BufferedMutator;
import org.apache.hadoop.hbase.client.BufferedMutatorParams;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.HTableInterface;
import org.apache.hadoop.hbase.client.NoopRegistry;
import org.apache.hadoop.hbase.client.RegionLocator;
import org.apache.hadoop.hbase.security.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AliHBaseUEConnection implements Connection {
  private static final Logger LOG = LoggerFactory.getLogger(AliHBaseUEConnection.class);


  private Configuration conf;


  private boolean isClosed = false;
  private boolean isAborted = false;

  private AliHBaseImplFactory factory;


  private int operationTimeout;


  // For HBase-1.x
  public AliHBaseUEConnection(Configuration conf, boolean managed, ExecutorService pool,
      final User user) throws IOException {
    this(conf, pool, user);
  }

  public AliHBaseUEConnection(Configuration conf, ExecutorService pool, final User user) throws IOException {
    this.conf = new Configuration(conf);
    // disable RegionSizeCalculator which is not supported when using MR
    this.conf.setBoolean("hbase.regionsizecalculator.enable", false);
    // use for compatibility client, we need to stop create AliHBaseUTTable/Admin inside
    // HTable or HBaseAdmin if it is created by AliHBaseUTTable/Admin itself.
    this.conf.setBoolean(AliHBaseConstants.STOP_FAKE_HBASE_CLIENT, true);
    this.conf.set("hbase.client.registry.impl", NoopRegistry.class.getName());
    this.operationTimeout = conf.getInt(HConstants.HBASE_CLIENT_OPERATION_TIMEOUT,
        HConstants.DEFAULT_HBASE_CLIENT_OPERATION_TIMEOUT);
    this.factory = AliHBaseImplFactory.create(conf);
  }

  public AliHBaseAPIProxy getAPIProxy(TableName tableName) throws IOException {
    return factory.getHBaseAPIProxyImpl(tableName);
  }


  @Override
  public Configuration getConfiguration() {
    return conf;
  }

  @Override
  public BufferedMutator getBufferedMutator(TableName tableName) throws IOException {
    return new AliHBaseUEBufferedMutator(AliHBaseUEConnection.this, tableName);
  }

  @Override
  public BufferedMutator getBufferedMutator(BufferedMutatorParams params)
      throws IOException {
    if (params.getTableName() == null) {
      throw new IllegalArgumentException("TableName cannot be null.");
    } else {
      return new AliHBaseUEBufferedMutator(AliHBaseUEConnection.this, params.getTableName());
    }
  }

  @Override
  public RegionLocator getRegionLocator(TableName tableName) throws IOException {
    return new AliHBaseUETable(tableName, AliHBaseUEConnection.this);
  }

  @Override
  public Admin getAdmin() throws IOException {
    return new AliHBaseUEAdmin(this);
  }

  @Override
  public synchronized void close() throws IOException {
    if (isClosed) {
      return;
    }
    this.factory.close();
    this.isClosed = true;
  }

  @Override
  public boolean isClosed() {
    return isClosed;
  }

  @Override
  public HTableInterface getTable(TableName tableName) throws IOException {
    return new AliHBaseUETable(tableName, AliHBaseUEConnection.this);
  }

  public HTableInterface getTable(String tableName) throws IOException {
    return getTable(TableName.valueOf(tableName));
  }

  @Override
  public HTableInterface getTable(TableName tableName, ExecutorService executorService) throws IOException {
    return getTable(tableName);
  }

  @Override
  public void abort(String s, Throwable throwable) {
    isAborted = true;
    try {
      close();
    } catch (IOException ioE) {
      LOG.error("Error happened when aborting", ioE);
    }
  }

  @Override
  public boolean isAborted() {
    return isAborted;
  }

  public int getOperationTimeout() {
    return operationTimeout;
  }

  public AliHBaseImplFactory getFactory() {
    return factory;
  }
}
